/* MMU Functions
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Image Loader.

The Imote2 Image Loader is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

The Imote2 Image Loader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the Imote2 Image Loader. If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CORE_MMU_H
#define CORE_MMU_H

#include <inttypes.h>

/** \brief enable the Memory Management Unit (MMU)
 * \details disables the instruction cache, sets the translation table base,
 *  activates the mmu and reenables the instruction cache (if it was enabled
 *  before)
 * \param tbl translation table base
 * \note set the domain access control register first using mmuSetDomain()
 * \note look at the Intel XScale Core Manual on Page 3.4.1-42 (MMU) and
 *  7.2.2-83 (Control Register) */
EXTERN_C void mmuEnable(const void * tbl);

/** \brief determine wether the MMU is enabled
 * \return \c TRUE iff the MMU is enabled
 * \note look at the Intel XScale Core Manual on Page 7.2.2-83 (Control
 *  Register) */
EXTERN_C bool mmuIsEnabled();

/** \brief disable the Memory Management Unit (MMU)
 * \details disables the instruction cache, branch target buffer and data
 *  cache, deactivates the mmu and reenables the instruction cache (if it was
 *  enabled before)
 * \note look at the Intel XScale Core Manual on Page 3.4.1-42 (MMU) and
 *  7.2.2-83 (Control Register) */
EXTERN_C void mmuDisable();

/** \brief set the control value of a domain in the access control register
 * \param domain the domain to be set
 * \param value the access value to be set for the domain
 * \note look at the Intel XScale Core Manual on Page 7.2.4-85
 *  (Domain Access Control Register) and the ARM Architecture Reference Manual
 *  on Section B4.3.2 */
EXTERN_C void mmuSetDomain(uint32_t domain, uint32_t value);

/** \brief set all domains at once
 * \param domains domain values
 * \note look at the Intel XScale Core Manual on Page 7.2.4-85
 *  (Domain Access Control Register) and the ARM Architecture Reference Manual
 *  on Section B4.3.2 */
EXTERN_C void mmuSetDomains(uint32_t domains);

/** \brief get the domain access control register
 * \return value of the domain access control register
 * \note look at the Intel XScale Core Manual on Page 7.2.4-85
 *  (Domain Access Control Register) */
EXTERN_C uint32_t mmuGetDomain();

/** \brief invalidate instruction tlb
 * \note this won't affect locked entries. use iTlbUnlock() if needed
 * \note look at the Intel XScale Core Manual on Page 3.4.1-42 (MMU) and
 *  7.2.9-89 (TLB Operations) */
EXTERN_C void iTlbInvalidate();

/** \brief invalidate data tlb
 * \note this won't affect locked entries. use dTlbUnlock() if needed
 * \note look at the Intel XScale Core Manual on Page 3.4.1-42 (MMU) and
 *  7.2.9-89 (TLB Operations) */
EXTERN_C void dTlbInvalidate();

/** \brief invalidate instruction and data tlb
 * \note this won't affect locked entries. use iTlbUnlock() and dTlbUnlock()
 *  if needed
 * \note look at the Intel XScale Core Manual on Page 3.4.1-42 (MMU) and
 *  7.2.9-89 (TLB Operations) */
EXTERN_C void iAndDTlbInvalidate();

/** \brief lock an entry in the instruction tlb
 * \param addr VMA of entry to be locked
 * \note look at the Intel XScale Core Manual on Page 3.4.3-43 (MMU) and
 *  7.2.11-91 (TLB Lock Down) */
EXTERN_C void iTlbLockLine(const void * addr);

/** \brief lock an entry in the data tlb
 * \param addr VMA of entry to be locked
 * \note look at the Intel XScale Core Manual on Page 3.4.3-44 (MMU) and
 *  7.2.11-91 (TLB Lock Down) */
EXTERN_C void dTlbLockLine(const void * addr);

/** \brief unlock instruction tlb
 * \note look at the Intel XScale Core Manual on Page 3.4.3-43 (MMU) and
 *  7.2.11-91 (TLB Lock Down) */
EXTERN_C void iTlbUnlock();

/** \brief unlock data tlb
 * \note look at the Intel XScale Core Manual on Page 3.4.3-44 (MMU) and
 *  7.2.11-91 (TLB Lock Down) */
EXTERN_C void dTlbUnlock();

#endif

